
import os
import ast


import logging

# Get the logger for 'httpx'
httpx_logger = logging.getLogger("httpx")

# Set the logging level to WARNING to ignore INFO and DEBUG logs
httpx_logger.setLevel(logging.WARNING)



from langchain_openai import ChatOpenAI

from crewai import Agent, Task, Crew, Process


from crewai_tools import tool

import hashlib



import inspect 

import time


try:
  from .utils.get_tiger_tools import get_tiger_tools
  from .utils.fix_string import fix_string
  from .utils.generate_file_tree import generate_file_tree
  from .utils.string_to_function import string_to_function
  from .utils.str_to_python_vars import str_to_python_var
  from .utils.notes import get_notes, add_note, reset_notes
  from .utils.old_functions import get_old_codes, get_old_functions, the_previously_function_codes, get_code_of_function, set_function_to_code_base, the_functions, get_function_index, reset_functions
  from .utils.prev_calls import save_args_decorator, get_previosly_last_return_of_near_element, get_previosly_last_call_of_near_element, reset_calls
  from .prompt_to_function import prompt_to_function
  from .utils.configs import prompt_as_config
except:
    from utils.get_tiger_tools import get_tiger_tools
    from utils.fix_string import fix_string
    from utils.generate_file_tree import generate_file_tree
    from utils.string_to_function import string_to_function
    from utils.str_to_python_vars import str_to_python_var
    from utils.notes import get_notes, add_note, reset_notes
    from utils.old_functions import get_old_codes, get_old_functions, the_previously_function_codes, get_code_of_function, set_function_to_code_base, the_functions, get_function_index, reset_functions
    from utils.prev_calls import save_args_decorator, get_previosly_last_return_of_near_element, get_previosly_last_call_of_near_element, reset_calls
    from prompt_to_function import prompt_to_function
    from utils.configs import prompt_as_config



def prompt_code(prompt, tester=False, custom_the_result=None, function_name=None):
    start_time = time.time()
    verbose_level = False
    if tester:
        print("Verbose activated")
        verbose_level = True

    function_start_time = time.time()
    the_result = prompt_to_function(prompt, tester=tester, more=True, function_name=function_name) if custom_the_result == None else custom_the_result
    function_end_time = time.time()
    the_tool_func = the_result["function"]
    the_tool = save_args_decorator(the_result["function"])
    the_releated_functions = the_result["related"]
    the_releated_codes = the_result["related_codes"]


        
    previosly_calls = ""
    for each_f in the_releated_functions:
        previosly_calls += "\n\nFunction:"+ each_f + "\nCALL" + get_previosly_last_call_of_near_element(each_f)

    llm = ChatOpenAI(model=prompt_as_config.OPENAI_MODEL, api_key=prompt_as_config.OPENAI_API_KEY, verbose=verbose_level)


    
    coder = Agent(
      role="Senior Python Developer with high level function as a service knowledge",
      goal="Running functions by request",
      backstory="You are graduated from Computer Science department of University and worked in biggest tech companies.",
      verbose=verbose_level,
      llm=llm
    )
    

    task_prompt = f"""
<USER>
{prompt} - please make this with {the_tool.__name__} function.
</USER>

<NOTES>
{get_notes()}
</NOTES> 

<OLD_CALLS>
{previosly_calls}
</OLD_CALLS>


<SYSTEM>
- Please read the notes for taking some important point like db paths, log locations and used libraries, db tables, etc.

- working directory (Dont use any other directory for any purpose) don't use sub folders if you need a file please specify above this:
{os.getcwd()}

- Think like a software developer and seperate the database tables, note them, create databases and somethings.

</SYSTEM>


DO USER REQUEST

"""
    


    the_tool = tool(the_tool)
    the_tool.func
    file_reader_tool = tool(generate_file_tree)

    
    task1 = Task(
      description=task_prompt,
      expected_output="""
RUNNED or NOT RUNNED
""",
      agent=coder,        
      tools=[the_tool, file_reader_tool, tool(add_note)]
    )



    class sub_class(Task):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.description = self.description_
    
        @property
        def description_(self):
            return f"""
<CALL>
{get_previosly_last_call_of_near_element(the_tool_func.__name__)}
</CALL>

<USER>
Extract some important points like table names, file locations and decisions. 
After that take notes via add_note tool.
If nothing important just pass don't add any note.
Dont take notes for requlary things
</USER>
"""

    
    task2 = sub_class(
      description="",
      expected_output="""
IF NOTES LEN BIGGER THAN 0
<TOOK_NOTES>
-
</TOOK_NOTES>   
ELSE
<TOOK_NOTES>
PASS
</TOOK_NOTES>   
""",
      agent=coder,
      tools=[tool(add_note)]
    )
    
    crew = Crew(
      agents=[coder],
      manager_llm=llm,
      tasks=[task1, task2],
      verbose=verbose_level,     
      share_crew=False
    )
    
    
    result = crew.kickoff()
    the_result = get_previosly_last_return_of_near_element(the_tool_func.__name__)
    end_time = time.time()

    function_time = int(function_end_time - function_start_time)
    print("FUNCTION TIME", function_time)
    total_time = int(end_time - start_time)
    interpreting_time = total_time - function_time
    print("INTERPRETING TIME", interpreting_time)
    print("TOTAL TIME", total_time)
    return the_result
























from functools import wraps
import inspect

try:
    from .prompt_to_code import prompt_code
    from .prompt_to_function import prompt_to_function
except:
    from prompt_to_code import prompt_code
    from prompt_to_function import prompt_to_function

def pf(*args, tester=False, function_just=False):
    def decorator(func):
        @wraps(func)
        def real_decorator(*inner_args, **inner_kwargs):
            print("name: ", func.__name__)
            print("args: ", inner_args)
            print("kwargs: ", inner_kwargs)
            print("explanation: ", func.__doc__)
            
            
            training_prompt = (
                    f"That named as '{func.__name__}'. USE THIS NAME AS FUNCTION NAME. "
                    f"And the signature should be: {inspect.signature(func)}. "
                    f"It is being called with the arguments {inner_args} and keyword arguments {inner_kwargs}. "
                    f"And make this operation with args: {func.__doc__}"
                )
            if tester:
                print(training_prompt)
            if not function_just:
                result = prompt_code(str(training_prompt), tester=tester, function_name=func.__name__)
            else:
                result = prompt_to_function(str(training_prompt), tester=tester, function_name=func.__name__)
            return result
        
        return real_decorator
    
    # Allows usage as either a plain decorator (without parentheses)
    # or a decorator factory (with parentheses).
    if args and callable(args[0]):
        return decorator(args[0])
    else:
        return decorator
    





#!/usr/bin/python3
# -*- coding: utf-8 -*-

from setuptools import setup
import platform


# Read the requirements from the requirements.txt file
with open("requirements.txt") as fp:
    install_requires = fp.read().splitlines()




setup(
    name="prompt_as",
    version="0.2.0",
    description="""GPT""",
    long_description="".join(open("README.md", encoding="utf-8").readlines()),
    long_description_content_type="text/markdown",
    url="https://github.com/Upsonic/prompt-as-function",
    author="Upsonic",
    author_email="onur@upsonic.co",
    license="MIT",
    packages=[
        "prompt_as",
    ],
    include_package_data=True,
    install_requires=[],
    python_requires=">= 3.9",
    zip_safe=False,

)




from langchain_openai import ChatOpenAI
from crewai import Agent, Task, Crew, Process
import hashlib

try:
  from .utils.old_functions import get_function_index, get_code_of_function, set_function_to_code_base, get_old_functions, get_old_codes, the_functions, the_previously_function_codes
  from .utils.string_to_function import string_to_function
  from .utils.get_tiger_tools import get_tiger_tools
  from .utils.fix_string import fix_string
  from .utils.notes import get_notes
  from .utils.configs import prompt_as_config
except:
  from utils.old_functions import get_function_index, get_code_of_function, set_function_to_code_base, get_old_functions, get_old_codes, the_functions, the_previously_function_codes
  from utils.string_to_function import string_to_function
  from utils.get_tiger_tools import get_tiger_tools
  from utils.fix_string import fix_string
  from utils.notes import get_notes
  from utils.configs import prompt_as_config



def prompt_to_function(prompt:str, tester:bool=False, more=False, function_name=None):
    print("Started for: ", prompt)
    global the_previously_function_codes
    verbose_level = False
    if tester:
        verbose_level = True
    
    llm = ChatOpenAI(model=prompt_as_config.OPENAI_MODEL, api_key=prompt_as_config.OPENAI_API_KEY, verbose=verbose_level)


    # Executor agent for making executions for mission
    coder = Agent(
      role="Senior Python Developer with high level function as a service knowledge",
      goal="Writing functions by request",
      backstory="You are graduated from Computer Science department of University and worked in biggest tech companies.",
      verbose=verbose_level,
      allow_delegation=False,
      llm=llm
    )





                       
    
    pre_check_prompt = f"""
<SYSTEM>
Hi, we have a operation came from user before the start please check the currenty function index and select some functions for understanding system design
</SYSTEM>

<IDEA>
{prompt}
</IDEA>

<FUNCTION_INDEX>
{get_function_index()}
</FUNCTION_INDEX>

<USER>
Select some functions by IDEA to understand system design. You are free to learn more about system, save mechanism and anything that you want to know.
</USER>


DO USER REQUEST

"""



    def callback_function(output):
        print("The related functions: ", output.raw)
        global the_previously_function_codes
        description = hashlib.sha256(output.description.encode()).hexdigest()
        the_previously_function_codes[description] = {"code":"", "name":[]}
        for each_function in get_function_index():
            if each_function in output.raw:
                the_previously_function_codes[description]["code"] += "\n" + get_code_of_function(each_function)
                the_previously_function_codes[description]["name"].append(each_function)



    taskpre = Task(
      description=pre_check_prompt,
      expected_output="""
IF THERE RELEATED_FUNCTIONS LEN BIGGER THAN 0
<releated_functions_names>
- 
</releated_functions_names>
ELSE
<releated_functions_names>
pass
</releated_functions_names>
""",
      agent=coder,
      callback=callback_function
    )    
    print("\nNotes: ", get_notes())
    print("\nFunction index: ", get_function_index())



    

    class sub_class_(Task):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.description = self.description_
    
        @property
        def description_(self):
            return f"""
<SYSTEM>
Write a python function that makes  by the Function Plan

if there is a needed library then put into inside of function. 

Dont write any usage example. Just function please. 

Please prefer easy to use (prefer sqlite rather than psycopg2) libraries.

Please be careful about user function name suggestion. It should be exacly same otherwise user can be verry angry and fire all of us.
</SYSTEM>    

<USER_REQUEST>
{prompt}
</USER_REQUEST>

<NOTES>
{get_notes()}
</NOTES>

<OLD_FUNCTIONS>
{get_old_codes(pre_check_prompt)}
</OLD_FUNCTIONS>


<SYSTEM>
Please read the notes for taking some important point like db paths, log locations and used libraries, etc.
</SYSTEM>

"""
            
            
    
    task1 = sub_class_(
      description="",
      expected_output="""
<code>
The imports
The function
</code>
""",
      agent=coder,
    )

    task2 = Task(
      description=f"""
Please rewrite the function and please remove the localized things. 
Use parameters with annotation for localized things. 
If there is an parameter as object, rewrite the function with no parameter as object. 
BUT IF THE LOCALIZED THINGS DOESNT IMPORTANT FOR USER (Like online service URLs, static names and etc) DONT TRANSFORM IT. 
When you remove something please specify in docstring for an example

Dont change the function name.
""",
      expected_output="""
<code>
The imports
The function
</code>
""",
      agent=coder,        
      context=[task1],
    )    

    task3 = Task(
      description="""
<SYSTEM>
Dont write any other thing over than the requested output type. Just write the imports and the function after that and dont add any ``` or anything that not okey for python interpreter
</SYSTEM>

<USER>
Please rewrite the function with docstring, and be sure the output is in the requested format.
</USER>
""",
      expected_output="""
<code>
The imports
The function
</code>
""",
      agent=coder,        
      context=[task2],
    )    






    
    task4 = Task(
      description=f"""
<SYSTEM>
If there is needed library for the function, install it. Dont try to instal builtin library. Just install third party libraries.
</SYSTEM>
""",
      expected_output="""
<INSTALlLED_LIBRARIES>
- Library Name
</INSTALlLED_LIBRARIES>
""",
      tools=get_tiger_tools(),
      agent=coder,
      context=[task3]
    )        
    

    crew = Crew(
      agents=[coder],
      tasks=[taskpre, task1, task2, task3, task4],
      verbose=verbose_level,
      share_crew=False,  
    )
    

    if function_name == None or not function_name in the_functions:
        result = crew.kickoff()
    
    
        result = task3.output.raw
        
    
        result = fix_string(result)
        code_of_f = result

    


    

        
        print("\nGenerated Function: ", f"START{result}END")
        
        result = string_to_function(result)
    
        set_function_to_code_base(result.__name__, code_of_f)


    
        if not more:
            return result
        else:
            return {"function":result, "related": get_old_functions(pre_check_prompt), "related_codes": get_old_codes(pre_check_prompt)}
    else:
        print("USING OLD FUNCTION")
        result = the_functions[function_name]
        result = string_to_function(result)
        if not more:
            return result
        else:
            return {"function":result, "related": [function_name], "related_codes": ""}



